/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "classimportxml.h"
#include "class.h"
#include "operation.h"
#include "attribute.h"
#include "classparser/ClassParser.h"
#include "classparser/ParsedArgument.h"
#include "classparser/ParsedAttribute.h"
#include "classparser/ParsedClass.h"
#include "classparser/ParsedMethod.h"
#include <kdebug.h>
#include <qdom.h>
#include <iostream>
using namespace std;

ClassImportXML::ClassImportXML(QWidget *parent, const char *name) : UMLDoc(parent, name) {
}

ClassImportXML::~ClassImportXML() {}
/** No descriptions */

UMLObject *ClassImportXML::createUMLObject(QString className, Uml::UMLObject_Type type) {

	UMLObject * o = findUMLObject(type, className);

	if(!o) {
		if(type == Uml::ot_Class) {
			UMLClass *c = new UMLClass(this, className, ++uniqueID);
			o = (UMLObject*)c;
		} else {
			kdDebug() << "CreateUMLObject(int) error" << endl;
			return o;
		}

		objectList.append(o);
		emit sigObjectCreated(o);
		setModified(true);
		return o;
	}
	return o;//gustavo - I put it here after compiler warning
}

/** No descriptions */
void ClassImportXML::insertAttribute(UMLObject *o, Uml::Scope scope, QString name, QString type, QString value /*= ""*/) {
	int attID = ++uniqueID;

	UMLAttribute *temp = reinterpret_cast<UMLAttribute *>(((UMLClass*)o)->addAttribute(name , attID));

	temp->setTypeName(type);
	temp->setInitialValue(value);
	temp->setScope(scope);

	setModified(true);
}

/** No descriptions */
void ClassImportXML::insertMethod(UMLObject *o, Uml::Scope scope, QString name, QString type, QPtrList<UMLAttribute> *parList /*= NULL*/) {
	int attID = ++uniqueID;

	UMLOperation *temp = reinterpret_cast<UMLOperation *>(((UMLClass*)o)->addOperation(name , attID));
	temp->setReturnType(type);

	if(parList != NULL) {
		QPtrListIterator<UMLAttribute> it(*parList);
		for( ; it.current(); ++it ) {
			UMLAttribute *par = it.current();
			int parID = ++uniqueID;
			par->setID(parID);
			temp->addParm(par);
		}
	}

	temp->setScope(scope);
	setModified(true);
}

void ClassImportXML::importXML(QStringList headerFileList) {
	QStringList::Iterator fileIT = headerFileList.begin();
	for( ; fileIT != headerFileList.end(); ++fileIT) {
		importXMLFile( (*fileIT).latin1() );
	} // for
}

void ClassImportXML::importXMLFile(const QString &filename) {
	QDomDocument doc( "doc");
    QFile file( filename );
    if ( !file.open( IO_ReadOnly ) )
        return;
    if ( !doc.setContent( &file ) ) {
        file.close();
        return;
    }
    file.close();

    // print out the element names of all elements that are direct children
    // of the outermost element.
    QDomElement docElem = doc.documentElement();

    QDomElement classElem = docElem;	// class

	cerr << "class:" << classElem.tagName().latin1() << endl;
	UMLObject *currentClass = createUMLObject(classElem.attribute("name"), Uml::ot_Class);
	QDomNode attributeElem = classElem.firstChild();
	QDomNode nn = attributeElem.firstChild();
	// add attributes
    while( !nn.isNull() ) {
        QDomElement e = nn.toElement(); // try to convert the node to an element.
		QDomElement typeNode = e.namedItem("type").toElement();
		QDomElement nameNode = e.namedItem("name").toElement();
		QString scope = e.attribute("protection");
		Uml::Scope attrScope = Uml::Public;
		if(scope=="private")
			attrScope = Uml::Private;
		else if(scope=="protected")
			attrScope = Uml::Protected;
		else if(scope=="public")
			attrScope = Uml::Public;
		insertAttribute(currentClass, attrScope, nameNode.text(), typeNode.text(), "");
        nn = nn.nextSibling();
    }
	QDomNode methodElem = attributeElem.nextSibling();
	nn = methodElem.firstChild();
	// add attributes
    while( !nn.isNull() ) {
        QDomElement e = nn.toElement(); // try to convert the node to an element.
		QDomElement typeNode = e.namedItem("type").toElement();
		QDomElement nameNode = e.namedItem("name").toElement();
		QDomElement paramsNode = e.namedItem("parameters").toElement();
		QString scope = e.attribute("protection");
		Uml::Scope attrScope = Uml::Public;
		if(scope=="private")
			attrScope = Uml::Private;
		else if(scope=="protected")
			attrScope = Uml::Protected;
		else if(scope=="public")
			attrScope = Uml::Public;
		QPtrList<UMLAttribute> parList;
		QDomElement paramNode = paramsNode.firstChild().toElement();
		while(!paramNode.isNull())
		{
			QDomElement typeNode = paramNode.namedItem("type").toElement();
			QDomElement nameNode = paramNode.namedItem("name").toElement();
			UMLAttribute *parameter = new UMLAttribute(currentClass, nameNode.text(), 0);
			parameter->setTypeName(typeNode.text());
			parList.append(parameter);
			paramNode = paramNode.nextSibling().toElement();
		}

		insertMethod(currentClass, attrScope, nameNode.text(), typeNode.text(), &parList);
        nn = nn.nextSibling();
    }


/*	CClassParser classParser;


	QStrList *cList;
	cList = classParser.store.getSortedClassNameList();
	QStrListIterator it(*cList);

	for(; it.current();++it ) {
		UMLObject *currentClass;
		CParsedClass* currentParsedClass =  classParser.store.getClassByName(it);
		QPtrList<CParsedAttribute> *attributes = currentParsedClass->getSortedAttributeList();
		QPtrListIterator<CParsedAttribute> aIt(*attributes);

		currentClass = createUMLObject(currentParsedClass->name, Uml::ot_Class);

		for( ; aIt.current() ; ++aIt) {
			CParsedAttribute *attr = aIt.current();
			QString scope;
			Uml::Scope attrScope = Uml::Public;
			switch(attr->exportScope) {
				case PIE_PUBLIC:
					scope = "public";
					attrScope = Uml::Public;
					break;

				case PIE_PROTECTED:
					scope = "protected";
					attrScope = Uml::Protected;

					break;


				case PIE_PRIVATE:
					scope = "private";
					attrScope = Uml::Private;
					break;
				case PIE_GLOBAL:
					break;
			} //switch
			this->insertAttribute(currentClass, attrScope, attr->name, attr->type, "");
		} // attribute for() loop

		//CParsedMethod *aMethod;
		QPtrList<CParsedMethod> *methods = currentParsedClass->getSortedMethodList();
		QPtrListIterator<CParsedMethod> mIt(*methods);

		for(;mIt.current(); ++mIt) {
			CParsedMethod *pMethod = mIt.current();
			QPtrListIterator<CParsedArgument> argsIt(pMethod->arguments);
			QString scope = "";
			Uml::Scope attrScope = Uml::Public;
			switch(pMethod->exportScope) {
				case PIE_PUBLIC:
					scope = "public";
					attrScope = Uml::Public;
					break;

				case PIE_PROTECTED:
					scope = "protected";
					attrScope = Uml::Protected;
					break;

				case PIE_PRIVATE:
					scope = "private";
					attrScope = Uml::Protected;
					break;
				case PIE_GLOBAL:
					break;
			} //switch

			QPtrList<UMLAttribute> parList;

			for( ; argsIt.current(); ++argsIt) {
				CParsedArgument *parg = argsIt.current();
				UMLAttribute *parameter = new UMLAttribute(currentClass, parg->name, 0);
				parameter->setTypeName(parg->type);
				parList.append(parameter);
			} //for( ; argsIt.current(); ++argsIt)

			QString methodType;
			if(!pMethod->isVirtual) {
				methodType = pMethod->type;
			} else {
				methodType = "virtual "+pMethod->type;
			}
			this->insertMethod(currentClass, attrScope, pMethod->name, methodType, &parList);
		} // method for() loop
	} // 	class for() loop*/
} // method

/** No descriptions */
bool ClassImportXML::createClass(QString className, UMLObject *uObject) {
	uObject = this->createUMLObject(className, Uml::ot_Class); // create class
	if(uObject != NULL)
		return true;
	else {
		return false;
	}
}
