<?xml version="1.0" encoding="UTF-8"?>
<xsl:transform version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" media-type="text/xml" />

	<xsl:template match="/">
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="doxygen">
		<xsl:apply-templates />
	</xsl:template>
	
	<xsl:template match="compounddef">
		<class>
			<xsl:attribute name="name">
				<xsl:value-of select="compoundname" />
			</xsl:attribute>
			<!-- Define a new order -->
			<attributes>
				<xsl:apply-templates mode="private-attributes"/>
				<xsl:apply-templates mode="protected-attributes"/>
				<xsl:apply-templates mode="public-attributes"/>
			</attributes>
			<methods>
				<xsl:apply-templates mode="private-methods"/>
				<xsl:apply-templates mode="protected-methods"/>
				<xsl:apply-templates mode="public-methods"/>
			</methods>
		</class>
	</xsl:template>

<!-- Here are the Attributes -->	
	<xsl:template match="sectiondef[@kind='private-attrib']" mode="private-attributes">
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="sectiondef[@kind='protected-attrib']" mode="protected-attributes">
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="sectiondef[@kind='public-attrib']" mode="public-attributes">
		<xsl:apply-templates />
	</xsl:template>


<!-- Here are the Methods -->
	<xsl:template match="sectiondef[@kind='private-func']" mode="private-methods">
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="sectiondef[@kind='protected-func']" mode="protected-methods">
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="sectiondef[@kind='public-func']" mode="public-methods">
		<xsl:apply-templates />
	</xsl:template>


<!-- this is a variable -->
	<xsl:template match="memberdef[@kind='variable']">
		<attribute>
			<xsl:attribute name="protection">
				<xsl:value-of select="@prot" />
			</xsl:attribute>				
			<type><xsl:value-of select="type" /></type>
			<name><xsl:value-of select="name" /></name>
		</attribute>
	</xsl:template>


<!-- this is a method -->
	<xsl:template match="memberdef[@kind='function']">
		<method>
			<xsl:attribute name="protection">
				<xsl:value-of select="@prot" />
			</xsl:attribute>				
			<type><xsl:value-of select="type" /></type>
			<name><xsl:value-of select="name" /></name>
			<parameters>
				<xsl:apply-templates />
			</parameters>
		</method>
	</xsl:template>


<!-- This is a parameter -->
	<xsl:template match="param">
		<param>
			<type><xsl:value-of select="type" /></type>
			<name><xsl:value-of select="declname" /></name>
		</param>
	</xsl:template>


<!-- The rest goes to nirwana -->
	<xsl:template match="text()|@*" />
	<xsl:template match="text()|@*" mode="private-attributes" />
	<xsl:template match="text()|@*" mode="protected-attributes"/>
	<xsl:template match="text()|@*" mode="public-attributes"/>
	<xsl:template match="text()|@*" mode="private-methods"/>
	<xsl:template match="text()|@*" mode="protected-methods"/>
	<xsl:template match="text()|@*" mode="public-methods"/>

</xsl:transform>
